/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

function PageControl(page) {
    var self = this,
        pageNow = "",
        pageNowIndex = -1,
        pageMain = "",
        pageMainIndex = -1,
        pageList = [],
        pageHistory = [];

    if (page) {
        pageNow = page;
        pageNowIndex = 0;
        pageMain = page;
        pageMainIndex = 0;
        pageList.push(page);
    }

    /**
     * Push a page to the history list.
     * @private
     * @param {string} page Name of the page to be pushed.
     */
    function pushHistory(page) {
        pageHistory.push(page);
    }
    /**
     * Pop a page from the history list.
     * @private
     * @return {string} name of the popped page.
     */
    function popHistory() {
        var retPage;

        if (pageHistory.length > 0) {
            retPage = pageHistory.pop();
        } else {
            console.warn("ERROR: Failed to popHistory - PageHistory is Empty");

            return null;
        }

        return retPage;
    }
    /**
     * show selected page and hide all other pages.
     * @private
     * @param {string} page Name of the page to be displayed.
     * @return {boolean} true if the page is successfully displayed.
     */
    function showPage(page) {
        var i,
            destPage = document.querySelector("#" + page),
            objPage;

        if (destPage !== null) {
            for (i = 0; i < pageList.length; i++) {
                objPage = document.querySelector("#" + pageList[i]);
                objPage.style.display = "none";
            }

            destPage.style.display = "block";
            pageNow = page;
        } else {
            console.warn("ERROR: Page named " + page + " is not exist.");

            return false;
        }

        return true;
    }

    /**
     * Add a new page to the page list.
     * @public
     * @param {string} page The name of a page to be added.
     * @param {number} index The index of a page to be added.
     * @return {boolean} true if the page is successfully added.
     */
    this.addPage = function(page, index) {
        var objPage = document.querySelector("#" + page);

        if (objPage) {
            if (index) {
                pageList.splice(index, 0, page);
            } else {
                pageList.push(page);
                index = pageList.length;
            }
        } else {
            console.warn("ERROR: Failed to addPage - The page doesn't exist");

            return false;
        }

        if (pageMain === "") {
            pageMain = page;
            pageMainIndex = index;
        }

        return true;
    };
    /**
     * Remove a page from the page list.
     * @public
     * @param {string} page The name of the a page to be removed.
     * @return {boolean} true if the page is successfully removed.
     */
    this.removePage = function(page) {
        var pageIndex = pageList.indexOf(page);

        if (pageIndex > -1) {
            if (pageIndex === pageMainIndex) {
                pageMain = "";
                pageMainIndex = -1;
            }

            pageList.splice(pageIndex, 1);
        } else {
            console.warn("ERROR: Failed to removePage - The page doesn't exist in pageList");

            return false;
        }

        return true;
    };
    /**
     * Move to the selected page.
     * @public
     * @param {string} page The name of a page to be displayed.
     * @return {boolean} true if the page is successfully displayed.
     */
    this.movePage = function(dest) {
        var lastPage = pageNow;

        if (showPage(dest)) {
            pushHistory(lastPage);
            pageNow = dest;
            pageNowIndex = pageList.indexOf(pageNow);
        } else {
            return false;
        }

        return true;
    };
    /**
     * Move back to the last page of the history list.
     * @public
     * @return {boolean} true if the page is successfully displayed.
     */
    this.moveBackPage = function() {
        var beforePage = popHistory();

        if (beforePage !== null) {
            showPage(beforePage);
            pageNow = beforePage;
            pageNowIndex = pageList.indexOf(pageNow);
        } else {
            console.warn("ERROR: Failed to backPage - popHistory returned null");

            return false;
        }

        return true;
    };
    /**
     * Move to the previous page of the page list.
     * @public
     * @return {boolean} true if the page is successfully displayed.
     */
    this.movePrevPage = function() {
        if (pageNowIndex > 0) {
            self.movePage(pageList[pageNowIndex - 1]);
        } else {
            console.warn("ERROR: Failed to movePrevPage - There is no previous page");

            return false;
        }

        return true;
    };
    /**
     * Move to the next page of the page list.
     * @public
     * @return {boolean} true if the page is successfully displayed.
     */
    this.moveNextPage = function() {
        if (pageNowIndex < pageList.length - 1) {
            self.movePage(pageList[pageNowIndex + 1]);
        } else {
            console.warn("ERROR: Failed to moveNextPage - There is no next page");

            return false;
        }

        return true;
    };
    /**
     * Get the name of the current page.
     * @public
     * @return {string} The name of the current page.
     */
    this.getPageNow = function() {
        return pageNow;
    };
    /**
     * Get the boolean value of the current page is the main page or not.
     * @public
     * @return {boolean} true if the current page is the main page.
     */
    this.isMainPage = function() {
        return (pageNow === pageMain);
    };

}